package com.tanpu.community.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tanpu.common.constant.BizStatus;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.dao.entity.community.ThemeCheckDuplicateEntity;
import com.tanpu.community.dao.mapper.community.ThemeCheckDuplicateMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Service
public class ThemeTextCheckService {

    @Resource
    private ThemeCheckDuplicateMapper themeCheckDuplicateMapper;

    private final String regex = "[，。！？；;, ]";


    public void insert(String content, String themeId, String userId,Integer themeType) {
        themeCheckDuplicateMapper.deleteByThemeId(themeId);
        content = content.replaceAll(regex, "。");
        String[] split = content.split("。");
        for (int i = 0; i < split.length; i++) {
            getMD5(split[i]);
            insert(themeId, split[i], getMD5(split[i]), i + 1, split.length,userId);
        }
    }

    public void deleteByThemeId(String themeId){
        themeCheckDuplicateMapper.deleteByThemeId(themeId);
    }

    public void insert(String themeId, String partition, String partitionHash, Integer num, Integer total,String userId) {

        ThemeCheckDuplicateEntity build = ThemeCheckDuplicateEntity.builder()
                .themeId(themeId)
                .partitionText(partition)
                .partitionHash(partitionHash)
                .partitionNum(num)
                .totalParts(total)
                .userId(userId)
                .build();
        themeCheckDuplicateMapper.insert(build);
    }

    public boolean checkDuplicate(String content) {
        if (content.length()<15){
            return false;
        }
        content = content.replaceAll(regex, "。");
        String[] split = content.split("。");
        List<String> list = new ArrayList<>();
        for (int i = 0; i < split.length; i++) {
            list.add(getMD5(split[i]));
        }
        LambdaQueryWrapper<ThemeCheckDuplicateEntity> w = new LambdaQueryWrapper<ThemeCheckDuplicateEntity>().in(ThemeCheckDuplicateEntity::getPartitionHash, list)
                .eq(ThemeCheckDuplicateEntity::getDeleteTag, BizStatus.DeleteTag.tag_init)
                .groupBy(ThemeCheckDuplicateEntity::getThemeId);
        Integer check = themeCheckDuplicateMapper.check(w);
        if (check!=null && check*10>=split.length*8){
            return true;
        }
        return false;
    }


    public static String getMD5(String str) {
        try {
            // 生成一个MD5加密计算摘要
            MessageDigest md = MessageDigest.getInstance("MD5");
            // 计算md5函数
            md.update(str.getBytes());
            // digest()最后确定返回md5 hash值，返回值为8为字符串。因为md5 hash值是16位的hex值，实际上就是8位的字符
            // BigInteger函数则将8位的字符串转换成16位hex值，用字符串来表示；得到字符串形式的hash值
            return new BigInteger(1, md.digest()).toString(16);
        } catch (Exception e) {
            throw new BizException("MD5加密出现错误");
        }
    }


    public void insertInit(String content, String themeId,String userId, LocalDateTime createTime) {
        themeCheckDuplicateMapper.deleteByThemeId(themeId);

        content = content.replaceAll(regex, "。");
        String[] split = content.split("。");
        for (int i = 0; i < split.length; i++) {
            getMD5(split[i]);

            ThemeCheckDuplicateEntity build = ThemeCheckDuplicateEntity.builder()
                    .themeId(themeId)
                    .partitionText(split[i])
                    .partitionHash(getMD5(split[i]))
                    .partitionNum(i+1)
                    .totalParts(split.length)
                    .userId(userId)
                    .createTime(createTime)
                    .updateTime(createTime)
                    .build();
            themeCheckDuplicateMapper.insert(build);
        }
    }

}
